@@ -5,9 +5,13 @@ from ..utils.constants import ObjectConstantsForData
 
 
 class DataContainer:
-    """
-    class DataContainer
+    """class DataContainer
     store objects types
+
+    Args:
+
+    Returns:
+
     """
     _elout = None
     _elout_object = None
@@ -31,9 +35,13 @@ class DataContainer:
     @staticmethod
     def get_data(name):
         """
-        :param: name
 
-        :return: Object on the given name
+        Args:
+          name: 
+
+        Returns:
+          Object on the given name
+
         """
         if name == ObjectConstantsForData.ELEMENT:
             return DataContainer._elout
@@ -74,14 +82,16 @@ class DataContainer:
 
     @staticmethod
     def init_all_data_sources(binout, logger, dynasaur_def, volume_path):
-        """
-        init all data sources
-        :param: binout
-        :param: logger
-        :param: dynasaur definition
-        :param: binout path/directory
+        """init all data sources
+
+        Args:
+          binout: 
+          logger: 
+          dynasaur_def: 
+          volume_path: 
+
+        Returns:
 
-        :return:
         """
         DataContainer._secforc = Secforc(binout, logger, dynasaur_def)
         DataContainer._elout = Elout(binout, logger, dynasaur_def)
