@@ -7,20 +7,17 @@ from ..utils.constants import LOGConstants, JsonConstants
 class BinoutData(object):
 
     def __init__(self, binout, logger, dynasaur_definitions, name):
-        """
-        Initialization/constructor
-        1 ) initializes members
-
-        2 ) initialize the member required_data_channel_ids. List of all data_channel_ids in the calc_proc.def file ,
-          i.e. {"type":"ELEMENT","ID":"id1","array":["(0, axial)"]
+        """Initialization/constructor
+            1 ) initializes members
+    
+            2 ) initialize the member required_data_channel_ids. List of all data_channel_ids in the calc_proc.def file ,
+                i.e. {"type":"ELEMENT","ID":"id1","array":["(0, axial)"]
                     used to check if channel "axial" is part of type ELEMENT
 
-        :param: binout
-        :param: logger
-        :param: dynasaur definition object
-        :param: name (binout name)
+        Args:
+
+        Returns:
 
-        :return:
         """
 
         self._binout = binout
@@ -37,82 +34,89 @@ class BinoutData(object):
         self._clean_required_data_channel_ids()
 
     def get_time(self):
-        """
-        getter
+        """getter
+
+        Args:
+
+        Returns:
+          np.array: time column vector
 
-        :returns: time column vector
-        :rtype: np.array
         """
         return self._time
 
     def get_interpolated_time(self):
-        """
-        getter
+        """getter
+
+        Args:
+
+        Returns:
+          np.array: interpolated time as column vector
 
-        :returns: interpolated time as column vector
-        :rtype: np.array
         """
         return self._time_interp
 
-    def get_channels_ids_object_name(self, object_name, plugin_name):
+    def get_channels_ids_object_name(self, object_name, binout_data_type):
         """
-        :param object_name:
-        :type object_name:
-        :param plugin_name:
-        :type plugin_name:
 
-        :returns: time
-        :rtype: np.array
+        Args:
+          object_name: type object_name:
+          binout_data_type: type plugin_name:
+
+        Returns:
+          np.array: time
+
         """
-        return self._dynasaur_definitions.get_ids_from_name(object_name, self._name, plugin_name)
+        defined_ids = self._dynasaur_definitions.get_ids_from_name(object_name, self._name, binout_data_type)
+        available_ids = self.get_ids()
+        channel_ids = np.intersect1d(defined_ids, available_ids)
+
+        return channel_ids
 
     def get_ids(self):
-        """
-        getter
+        """getter
+
+        Args:
+
+        Returns:
+          np.array: ids as numpy array (np.uint32)
 
-        :return: ids as numpy array (np.uint32)
-        :rtype: np.array
         """
         return self._ids.astype(np.uint32)
 
     def _interp_time(self, time_):
-        """
-        method to interpolate the given time to decimal values
+        """method to interpolate the given time to decimal values
 
-        :param time_:
-        :type np.array:
+        Args:
+          time_: type np.array:
+
+        Returns:
+          np.array as column vector: interpolated time
 
-        :returns: interpolated time
-        :rtype: np.array as column vector
         """
 
-        if self._dynasaur_definitions.get_units().second() == 1000:  # means ms
-            end_time_ = np.round(time_[-1] - time_[0])  # assumed ms
-            assert (time_[0] < 0.02)
-        elif self._dynasaur_definitions.get_units().second() == 1:  # means s
-            end_time_ = np.round(time_[-1] - time_[0], 3)
-            # TODO: make assert even if the condition satisfied
-            # assert (time_[0] < 0.00001)
-        else:
-            assert False
+        time_si = time_.to_base_units()
+        end_time_ = np.round(time_si[-1] - time_si[0], 3)
 
         # check if the time_diff is already constant:
         # https://stackoverflow.com/questions/3844801/check-if-all-elements-in-a-list-are-identical
         # https://numpy.org/doc/stable/reference/generated/numpy.allclose.html
         time_diff = np.diff(time_)
-        if np.allclose(time_diff, np.tile([time_diff[0]], len(time_diff))):
+        if np.allclose(time_diff, np.tile([time_diff[0].magnitude], len(time_diff)) * self._dynasaur_definitions.get_units().time):
             return time_
 
         time_interp = np.linspace(time_[0], end_time_, len(time_))
         return time_interp
 
     def _set_required_data_channel_ids(self):
-        """
-         Function to initialize member self._required_data_channel_ids
+        """Function to initialize member self._required_data_channel_ids
           --> extracts the required channel_types from the calc_proc.def  (_criteria, _data_vis)
           i.e. calc_proc.def : "array": ["(0, time)"],
           self._required_data_channel_ids would be appended ["(0, time)"]
 
+        Args:
+
+        Returns:
+
         """
         for def_key in self._dynasaur_definitions.__dict__:
             if def_key == "_criteria":
@@ -124,13 +128,18 @@ class BinoutData(object):
                     self._get_data_from_json(self._dynasaur_definitions.__dict__[def_key][key]["y"])
 
     def _get_data_from_json(self, json_object):
-        """
-        recursive method to extract array
-
+        """recursive method to extract array
+        
         inner part of the json object can be :
             * value
             * strain_stress
             * array
+
+        Args:
+          json_object: 
+
+        Returns:
+
         """
         if 'function' in json_object.keys():  # expected name and params
             parameter_def = json_object['function']['param']
@@ -150,14 +159,16 @@ class BinoutData(object):
                 assert False
 
     def _clean_required_data_channel_ids(self):
-        """
-        clean required_data_channel_ids types, stripes each entry in self._required_data_channel_ids
+        """clean required_data_channel_ids types, stripes each entry in self._required_data_channel_ids
         calc_proc.def : ["(0, axial)"] --> ["axial"]
         
         self._required_data_channel_ids is a casted to a set
-        
-        :param: None
-        :return: None
+
+        Args:
+
+        Returns:
+          None
+
         """
         _list_temp = []
         for array_temp in self._required_data_channel_ids:
@@ -181,18 +192,19 @@ class BinoutData(object):
         self._required_data_channel_ids = set(_list_temp)
 
     def _get_subdatatypes_to_reach_binout_data(self, ls, ls_subdatatypes):
-        """
-        Recursively traverses binout data and stores the required
+        """Recursively traverses binout data and stores the required
         subdatatypes to access data in  ls_subdatatypes
-
+        
         i.e. binout.read("bndout", "velocity", "rigidbody") --> ["ids", "x_force"]
           ls_subdatatypes --> [["bndout", "velocity", "rigidbody"]]
 
-        :param ls:
-            list of subdatatypes
-        :param ls_subdatatypes:
-            list of lists of subdatatyoes
-        :return: None
+        Args:
+          ls: list of subdatatypes
+          ls_subdatatypes: list of lists of subdatatyoes
+
+        Returns:
+          None
+
         """
         elements = self._binout.read(*ls)
         if self.__class__.ids_name in elements:
@@ -205,15 +217,15 @@ class BinoutData(object):
             self._get_subdatatypes_to_reach_binout_data(ls_copy, ls_subdatatypes)
 
     def _read_data_channels(self, subdatatypes=[]):
-        """
-        extracts the data_channels (conjunction of self.__class__.DATA_CHANNEL_TYPES and available in binout) and
+        """extracts the data_channels (conjunction of self.__class__.DATA_CHANNEL_TYPES and available in binout) and
         stores the interpolated data in self._data
 
-        :param subdatatypes: subdatatypes to access data
-        :type subdatatypes: list
+        Args:
+          subdatatypes(list, optional): subdatatypes to access data (Default value = [])
+
+        Returns:
+          None:: None:
 
-        :returns None:
-        :rtype None:
         """
         data = {}
         args_ = subdatatypes
@@ -275,8 +287,8 @@ class BinoutData(object):
             # type_data = np.reshape(type_data, (time_interp.shape[0], -1))
             data_interp = np.zeros(shape=(self._time_interp.shape[0], type_data.shape[1]))
             for i in range(type_data.shape[1]):
-                data_interp[:, i] = np.interp(self._time_interp,
-                                              self._time[:self._time_interp.shape[0]], type_data[:, i])
+                data_interp[:, i] = np.interp(self._time_interp.magnitude,
+                                              self._time.magnitude[:self._time_interp.shape[0]], type_data[:, i])
             data[key] = data_interp
 
         self._data = {**self._data, **data}
@@ -296,7 +308,7 @@ class BinoutData(object):
 
         assert self._ids is None and self._time is None and self._time_interp is None
 
-        self._time = self._binout.read(*subdatatypes, 'time').flatten()
+        self._time = self._binout.read(*subdatatypes, 'time').flatten() * self._dynasaur_definitions.get_units().time
         self._time_interp = self._interp_time(self._time)
         # TODO: Check if data is not there index out of range -> resulting crash
         self._ids = np.array(self._binout.read(*subdatatypes, self.__class__.ids_name))
@@ -308,19 +320,21 @@ class BinoutData(object):
         assert len(self._ids.shape) == 1
 
     def read_binout_data(self):
-        """
-        called from plugin classes (Controller) to initialize all BinoutData objects of the DataContainer (i.e. Glstat)
-
+        """called from plugin classes (Controller) to initialize all BinoutData objects of the DataContainer (i.e. Glstat)
+        
         __init_data__()
         * self._time
         * self._time_interp
         * self._ids
-
+        
         read in the data channels as defined in __class__.DATA_CHANNEL_TYPES
         _read_data_channels()
 
-        :returns: success on reading (True when data was read in, False when no data available)
-        :rtype: boolean value
+        Args:
+
+        Returns:
+          boolean value: success on reading (True when data was read in, False when no data available)
+
         """
         # data already read
         if self._ids is not None:
@@ -328,7 +342,6 @@ class BinoutData(object):
 
         # i.e.  no rcforc in binout
         if self._name not in self._binout.read():
-            self._logger.emit(LOGConstants.ERROR[0], "no " + self._name + " data in binout")
             return False
 
         # data read
@@ -356,45 +369,53 @@ class BinoutData(object):
         return True
 
     def _get_negative_intersection(self, data_channel_ids):
-        """
-        compares the required_data_channel_ids (as defined in the calc_proc.def) with the available
+        """compares the required_data_channel_ids (as defined in the calc_proc.def) with the available
         data_channel_ids of the given binout. returns a list of missing data_channel_ids
 
-        :param data_channel_ids:
-        :type data_channel_ids:
+        Args:
+          data_channel_ids: type data_channel_ids:
+
+        Returns:
+          list: negative_intersection: list of missing data_channel_ids
 
-        :returns negative_intersection: list of missing data_channel_ids
-        :rtype: list
         """
         # only necessary functions
         negative_intersection = [val for val in self._required_data_channel_ids if val not in data_channel_ids]
         return negative_intersection
 
-    def get_data_of_defined_json(self, json_object, data_offsets):
+    def get_data_of_defined_json(self, data_offsets, type: str, ID: str, array: list, **kwargs):
         """
-        :param json_object:
-        :type json_object:
-        :param data_offsets:
-        :type data_offsets:
 
-        :return: data array between data offset and date delta t
+        Args:
+            data_offsets: contains the object type and the first and last time dependent index that should be
+                       considered e.g. [('NODE', 0, 3000)]
+
+            type: dynasaur DataType as defined in utils.constants.DataTypeMapping (OBJECT, NODE, ...)
+
+            ID:   Object Identifier as defined in the object.def file
+
+            array: list of strings, consisting of  the data index in the objects definition file and the channel
+                    type e.g.  ["(0, internal_energy)"]
+
+
+        Returns:
+          data array between data offset and date delta t
+
         """
-        ids = self.get_channels_ids_object_name(json_object[JsonConstants.ID_UPPER_CASE], self._name)
+        ids = self.get_channels_ids_object_name(ID, self._name)
 
         if len(ids) == 0:
-            if "ID" in json_object:
-                self._logger.emit(LOGConstants.ERROR[0],
-                                  "Missing ID in binary input data, identifier: " + json_object["ID"])
+            self._logger.emit(LOGConstants.ERROR[0], "Missing ID in binary input data, identifier: " + ID)
             return
 
         data_offset = 0
         data_delta_t = -1
         for (t, offset, delta_t) in data_offsets:
-            if t == json_object["type"]:
+            if t == type:
                 data_offset = offset
                 data_delta_t = delta_t
 
-        array_definition = json_object[JsonConstants.ARRAY]
+        array_definition = array
         if array_definition[0].split(',')[0].strip(' (') == "all":
             converted_tuples = [(index, array_definition[0].split(',')[1].strip(' )')) for index, id_ in
                                 enumerate(ids)]
@@ -408,7 +429,7 @@ class BinoutData(object):
         for tpl in converted_tuples:
             d = copy.copy(self.get_measurement_channel(id_=ids[tpl[0]], channel_name=tpl[1]))
             if len(d) == 0:
-                return None
+                return np.NaN
             if tpl[1] == "time":
                 d -= d[data_offset, 0]
             data_array = d[data_offset:data_delta_t] if data_array is None else np.append(data_array,
@@ -417,19 +438,94 @@ class BinoutData(object):
 
         return data_array
 
-    def get_measurement_channel(self, id_, channel_name):
+    def _get_unit_dimension_from_channel_name(self, channel_name):
         """
-        Returns the interpolated measurement_channel, defined through
+
+        :param channel_name:
+        :return:
+        """
+
+        dim_less = self._dynasaur_definitions.get_units().dimensionless
+        t = self._dynasaur_definitions.get_units().time
+        m = self._dynasaur_definitions.get_units().mass
+        l = self._dynasaur_definitions.get_units().length
+
+        force = m * l / (t**2)
+        moment = force * l
+        energy = force * l
+        area = l**2
+        acceleration = l/(t**2)
+        velocity = l/t
+        r_velocity = 1/t
+        r_acceleration = 1/(t**2)
+
+        if "forc" in channel_name or "axial" in channel_name or "radial" in channel_name:
+            return force
+
+        elif "moment" in channel_name:
+            return moment
+
+        elif "energy" in channel_name:
+            return energy
+
+        elif "area" in channel_name:
+            return area
+
+        # mind order of if
+        elif "rx_acceleration" in channel_name or "ry_acceleration" in channel_name or "rz_acceleration" in channel_name:
+            return r_acceleration
+
+        elif "acceleration" in channel_name:
+            return acceleration
+
+        # mind order of if
+        elif "rx_velocity" in channel_name or "ry_velocity" in channel_name or "rz_velocity" in channel_name:
+            return r_velocity
+
+        elif "velocity" in channel_name:
+            return velocity
+
+        # mind order of if
+        elif "rx_displacement" in channel_name or "ry_displacement" in channel_name or "rz_displacement" in channel_name:
+            return dim_less
+
+        elif "displacement" in channel_name:
+            return l
+
+        elif "position" in channel_name:
+            return l
+
+        elif "coordinate" in channel_name:
+            return l
+
+        elif "length" in channel_name:
+            return l
+
+        elif "time" in channel_name:
+            return t
+
+        elif "mass" in channel_name:
+            return m
+
+        else:
+            return self._dynasaur_definitions.get_units().dimensionless
+
+    def get_measurement_channel(self, id_, channel_name):
+        """Returns the interpolated measurement_channel, defined through
           - id_             (identifier in ids.def files)
           - channel_name    ("array" in calc_proc.def )  Aim: Constant time interval for filter operations.
 
-        :param id_:
-        :type id_: str_ or int:
-        :param channel_name:
-        :type channel_name: str
-        :returns: the interpolated channel
-        :rtype: np.array
+        Args:
+          id_: type id_: str_ or int:
+          channel_name: type channel_name: str
+
+        Returns:
+          np.array: the interpolated channel
+
         """
+
+        unit = self._get_unit_dimension_from_channel_name(channel_name)
+
         if channel_name == 'time':
             return self._time_interp.reshape(-1, 1)
 
@@ -440,7 +536,7 @@ class BinoutData(object):
 
         if len(data_index) == 0:
             self._logger.emit(LOGConstants.ERROR[0], 'ID ' + str(id_) + ' not in binout')
-            exit()
+            return []
 
         self._logger.emit(LOGConstants.DATA_PLUGIN[0], 'read id ' + str(id_) + ' from channel name: ' + channel_name)
         assert (len(data_index) >= 1)
@@ -451,4 +547,4 @@ class BinoutData(object):
 
         interpolated_channel = self._data[channel_name][:, data_index]
 
-        return interpolated_channel.reshape(-1, len(data_index))
+        return interpolated_channel.reshape(-1, len(data_index)) * unit
