@@ -1,25 +1,28 @@
 import time
 import json
+
 from lasso.dyna.Binout import Binout
 
-
-from ..utils.constants import LOGConstants, DefFileConstants, UnitsConstants, DefinitionConstants, \
-    ObjectConstantsForData, JsonConstants, LoggerERROR, LoggerWARNING, PluginsParamDictDef, DataTypeMapping
+from ..utils.constants import LOGConstants, DefinitionConstants, ObjectConstantsForData, JsonConstants, LoggerERROR, \
+                              LoggerWARNING, PluginsParamDictDef, DataTypeMapping
 from ..utils.constants import LoggerReadDynasaurDefinitionFile as LD
-from ..data.def_file_validator import TestDefinitionJSON
+from ..utils.units import Units
 
 
 class Helpers(object):
+    """ """
 
     @staticmethod
     def create_default_object_definitions(data_source, code_type):
-        """
-        TODO
+        """TODO
         documentation
 
-        :param data_source:
-        :param code_type:
-        :return:
+        Args:
+          data_source: param code_type:
+          code_type: 
+
+        Returns:
+
         """
         d = {"OBJECTS":[]}
         if code_type == "LS-DYNA":
@@ -45,61 +48,64 @@ class Helpers(object):
         return d
 
 
-
-class Units(object):
-    def __init__(self):
-        self._time = 1.0  # in seconds
-        self._length = 1.0  # in meters
-        self._weight = 1.0  # in kilogram
-        self.grav_const = 9.81
-
-    def set_time(self, _time):
-        self._time = _time
-
-    def set_length(self, length):
-        self._length = length
-
-    def set_weight(self, weight):
-        self._weight = weight
-
-    def second(self):
-        return self._time
-
-    def meter(self):
-        return self._length
-
-    def kilogram(self):
-        return self._weight
-
-
 class DefintionObject(object):
     def __init__(self, oid):
-        """
-        Init function / constructor
-        :param oid:
-        :return:
+        """Init function / constructor
+
+        Args:
+            oid: return:
+
+        Returns:
+
         """
         self._oid = oid
         self._parts = {}
         self._elements = {}
 
     def set_parts(self, part):
+        """
+
+        Args:
+          part: 
+
+        Returns:
+
+        """
         self._parts = part
 
     def add_part(self, pid):
+        """
+
+        Args:
+          pid: 
+
+        Returns:
+
+        """
         self._parts[pid] = pid
 
     def add_element(self, eid):
+        """
+
+        Args:
+          eid: 
+
+        Returns:
+
+        """
         self._elements[eid] = eid
 
     def get_parts(self):
+        """ """
         return sorted(self._parts)
 
     def get_element(self):
+        """ """
         return sorted(self._elements)
 
 
 class DynasaurDefinitions:
+    """ """
     # create the object container
     def __init__(self, logger):
         """
@@ -124,15 +130,22 @@ class DynasaurDefinitions:
     def binout_type_data_type_container(self, binout_name):
         """
 
-        :param binout_name:
-        :return:
+        Args:
+          binout_name: return:
+
+        Returns:
+
         """
         return DataTypeMapping.binary_identifier_2_dynasaur_identifier(binout_name) in self._dynasaur_data_type_container
 
     def get_param_dict_from_command(self, command):
         """
-        :param command:
-        :return dictionary on the command
+
+        Args:
+          command: return dictionary on the command
+
+        Returns:
+
         """
         if 'visualization' in command:
             d = command
@@ -148,23 +161,21 @@ class DynasaurDefinitions:
             return
 
     def get_units(self):
-        """
-        get units
-
-        :param:
-        :return units:
-        """
+        """get units"""
         return self._units
 
     def get_criteria_ids(self):
+        """ """
         return [key for key in self._criteria.keys()]
 
     def get_criteria(self, key):
-        """
-        get criteria
+        """get criteria
+
+        Args:
+          key: return criteria:
+
+        Returns:
 
-        :param key:
-        :return criteria:
         """
         if key not in self._criteria.keys() or len(self._criteria) == 0:
             self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[6] + "\"" + key + "\"")
@@ -173,10 +184,7 @@ class DynasaurDefinitions:
         return self._criteria[key]
 
     def get_data_vis_calc_commands(self):
-        """
-
-        :return:
-        """
+        """:return:"""
 
         ls = [key for key in self._data_vis.keys()]
         ret = []
@@ -188,11 +196,18 @@ class DynasaurDefinitions:
             if "label" in self._data_vis[key]["y"]:
                 y_label = self._data_vis[key]["y"]["label"]
                 d["y_label"] = y_label
+            if "unit" in self._data_vis[key]["x"]:
+                x_label = self._data_vis[key]["x"]["unit"]
+                d["x_unit"] = x_label
+            if "unit" in self._data_vis[key]["y"]:
+                y_label = self._data_vis[key]["y"]["unit"]
+                d["y_unit"] = y_label
 
             ret.append(d)
         return ret
 
     def get_criteria_calc_commands(self):
+        """ """
         ls = [key for key in self._criteria.keys()]
         ret = []
         for nr, key in enumerate(ls):
@@ -201,45 +216,48 @@ class DynasaurDefinitions:
         return ret
 
     def get_data_vis(self, key):
-        """
-        get code
+        """get code
+
+        Args:
+          key: 
+
+        Returns:
 
-        :param:
-        :return code:
         """
         if key not in self._data_vis.keys() or len(self._data_vis) == 0:
             self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[7] + "\"" + key + "\"")
             exit("Exiting, check your def file")
         return self._data_vis[key]
 
-    def get_ids_from_name(self, name, data_container, plugin_name):
-        """
-        get ids from given name
+    def get_ids_from_name(self, name, data_container, binout_data_type):
+        """get ids from given name
+
+        Args:
+          name: param data_container:
+          binout_data_type: return dict(name):
+          data_container: 
+
+        Returns:
 
-        :param name:
-        :param data_container:
-        :param plugin_name:
-        :return dict(name):
         """
         dynasaur_data_type = DataTypeMapping.binary_identifier_2_dynasaur_identifier(data_container)
         d = self._dynasaur_data_type_container[dynasaur_data_type]
 
         if name not in d.keys() or len(d) == 0:
-            self._logger.emit(LOGConstants.ERROR[0],
-                              "In " + plugin_name + LoggerERROR.print_statements[5] + "\"" + name + "\"")
+            self._logger.emit(LOGConstants.ERROR[0], "\"" + name + "\" is not defined for " + plugin_name)
             time.sleep(.5)
-            exit("Exiting, check your def file")
-
-            return
+            return []
 
         return d[name]
 
     def get_defined_objects_containing_parts(self, part_ids):
-        """
-        get defined object containing parts
+        """get defined object containing parts
+
+        Args:
+          part_ids: return list_defined_objects:
+
+        Returns:
 
-        :param part_ids:
-        :return list_defined_objects:
         """
         list_defined_objects = []
         for tmp_object_name in sorted(self._objects):
@@ -254,11 +272,13 @@ class DynasaurDefinitions:
         return list_defined_objects
 
     def get_defined_objects_containing_all_parts(self, part_ids):
-        """
-        get defined object
+        """get defined object
+
+        Args:
+          part_ids: return list_defined_objects:
+
+        Returns:
 
-        :param part_ids:
-        :return list_defined_objects:
         """
         list_defined_objects = []
         for tmp_object_name in sorted(self._objects):
@@ -273,10 +293,13 @@ class DynasaurDefinitions:
         return list_defined_objects
 
     def define_dynasaur_everything(self, part_ids):
-        """
-        define dynasaur everything
-        :param part_ids:  list of part_ids
-        :return:
+        """define dynasaur everything
+
+        Args:
+          part_ids: list of part_ids
+
+        Returns:
+
         """
 
         # create an Everything object
@@ -287,12 +310,15 @@ class DynasaurDefinitions:
         self._objects[name].set_parts(part_ids)
 
     def get_all_data_types_from_json(self, d, ls):
-        """
-        get all data types
+        """get all data types
 
-        :param d:
-        :param ls:
+        Args:
+          d: param ls:
         :return d[TYPE]:
+          ls: 
+
+        Returns:
+
         """
 
         for key in d.keys():
@@ -310,11 +336,13 @@ class DynasaurDefinitions:
                 return d[JsonConstants.TYPE]
 
     def get_required_datatypes(self, plugin_name):
-        """
-        get required datatype
+        """get required datatype
+
+        Args:
+          plugin_name: return ls:
+
+        Returns:
 
-        :param plugin_name:
-        :return ls:
         """
         ls = []
         if DefinitionConstants.CRITERIA == plugin_name:
@@ -326,30 +354,42 @@ class DynasaurDefinitions:
         return ls
 
     def get_parts_by_object_containing_part_ids(self, tmp_object, part_ids):
-        """
-        get parts by object containing part ids
+        """get parts by object containing part ids
 
-        :param tmp_object:
-        :param part_ids:
+        Args:
+          tmp_object: param part_ids:
         :return parts by object:
+          part_ids: 
+
+        Returns:
+
         """
         return [val for val in part_ids if val in self._objects[tmp_object].get_parts()]
 
-    def get_info(self):
+    def get_parts_of_defined_object(self, tmp_object):
         """
-        get info
 
-        :param:
-        :return info:
+        Args:
+          tmp_object: type  str:
+
+        Returns:
+          array:: the parts as defined in ids.def:
+
         """
+        return self._objects[tmp_object].get_parts()
+
+    def get_info(self):
+        """get info"""
         return self._info
 
     def _set_contact(self, json_object):
-        """
-        set contact object type
+        """set contact object type
+
+        Args:
+          json_object: return:
+
+        Returns:
 
-        :param json_object:
-        :return:
         """
         name = json_object[JsonConstants.NAME]
 
@@ -364,9 +404,12 @@ class DynasaurDefinitions:
     def _set_datacontainer_defintion(self, json_object, def_container):
         """
 
-        :param json_object:
-        :param def_container:
-        :return:
+        Args:
+          json_object: param def_container:
+          def_container: 
+
+        Returns:
+
         """
         name = json_object[JsonConstants.NAME]
 
@@ -380,6 +423,23 @@ class DynasaurDefinitions:
         if JsonConstants.PART_ID in json_object:
             def_container[name] = json_object[JsonConstants.PART_ID]
 
+    def is_object_defined(self, tmp_object):
+        """
+        checks if the tmp_object is defined in the objects.def file
+
+        Args:
+          tmp_object: ID string as defined in the calc_proc.def file
+          (i.e. object_data": {
+                            "type" : "OBJECT",
+                            "ID" : "Ribs",
+                            "strain_stress": "Strain"})
+
+        Returns:
+            True or False
+        """
+
+        return tmp_object in self._objects
+
     def __set_object(self, json_object):
         """
         set object
@@ -407,17 +467,21 @@ class DynasaurDefinitions:
                 self._objects[name].add_element(i)
 
     def _parse_objects(self, json_objects):
-        """
-        parse json objects and call setter
+        """parse json objects and call setter
+
+        Args:
+          json_object: return:
+          json_objects: 
+
+        Returns:
 
-        :param json_object:
-        :return:
         """
         for json_object in json_objects[DefinitionConstants.OBJECTS]:
 
             if json_object[JsonConstants.TYPE] not in self._dynasaur_data_type_container.keys():
                 self._logger.emit(LOGConstants.WARNING[0],
-                                  json_object[JsonConstants.TYPE] + LoggerWARNING.print_statements[1])
+                                  json_object[JsonConstants.TYPE] + LoggerWARNING.print_statements[1] +
+                                  str(", ".join(list(self._dynasaur_data_type_container.keys()))))
                 continue
 
             if json_object[JsonConstants.TYPE] == ObjectConstantsForData.ELEMENTOBJECT:
@@ -428,47 +492,27 @@ class DynasaurDefinitions:
                 data_type_container = self._dynasaur_data_type_container[json_object[JsonConstants.TYPE]]
                 self._set_datacontainer_defintion(json_object, data_type_container)
 
-    def _set_unit(self, json_object):
-        """
-        set unit object
-
-        :param json_object:
-        :return:
-        """
-        for key_name in json_object[DefinitionConstants.UNIT]:
-            if key_name == UnitsConstants.TIME:
-                if json_object[DefinitionConstants.UNIT][key_name] == UnitsConstants.SECOND:
-                    self._units.set_time(UnitsConstants.ONE)
-                elif json_object[DefinitionConstants.UNIT][key_name] == UnitsConstants.MILLISECOND:
-                    self._units.set_time(UnitsConstants.THOUSAND)
-            elif key_name == UnitsConstants.LENGTH:
-                if json_object[DefinitionConstants.UNIT][key_name] == UnitsConstants.METER:
-                    self._units.set_length(UnitsConstants.ONE)
-                elif json_object[DefinitionConstants.UNIT][key_name] == UnitsConstants.MILLIMETER:
-                    self._units.set_length(UnitsConstants.THOUSAND)
-            elif key_name == UnitsConstants.WEIGHT:
-                if json_object[DefinitionConstants.UNIT][key_name] == UnitsConstants.KILOGRAM:
-                    self._units.set_weight(UnitsConstants.ONE)
-                elif json_object[DefinitionConstants.UNIT][key_name] == UnitsConstants.TON:
-                    self._units.set_weight(UnitsConstants.WEIGHT_DEFAULT)
-
     def _set_criteria(self, json_object):
-        """
-        set criteria object
+        """set criteria object
+
+        Args:
+          json_object: return:
+
+        Returns:
 
-        :param json_object:
-        :return:
         """
         for criteria in json_object[DefinitionConstants.CRITERIA]:
             body_part = criteria[JsonConstants.PART_OF]
             self._criteria[body_part + "_" + criteria[JsonConstants.NAME]] = criteria
 
     def _set_diagram_visualisation(self, json_object):
-        """
-        set data visualisation object
+        """set data visualisation object
+
+        Args:
+          json_object: return:
+
+        Returns:
 
-        :param json_object:
-        :return:
         """
         for definition in json_object[DefinitionConstants.DATA_VIS]:
             body_part = definition[JsonConstants.PART_OF]
@@ -478,12 +522,7 @@ class DynasaurDefinitions:
                 self._logger.emit(LOGConstants.WARNING[0], "Double declaration of " + body_part + "_" + definition[JsonConstants.NAME])
 
     def _set_info(self):
-        """
-        set info in data visualisation
-
-        :param:
-        :return:
-        """
+        """set info in data visualisation"""
         info = []
         for key in self._data_vis.keys():
             if JsonConstants.INFO in self._data_vis[key].keys():
@@ -503,11 +542,13 @@ class DynasaurDefinitions:
         self._info = info
 
     def read_def(self, fn):
-        """
-        read definition file(s)
+        """read definition file(s)
+
+        Args:
+          fn: return:
+
+        Returns:
 
-        :param fn:
-        :return:
         """
         if fn is None:
             exit("No definition file")
@@ -538,11 +579,11 @@ class DynasaurDefinitions:
 
                                 elif DefinitionConstants.UNIT in j_object:
                                     if not is_entity_set[DefinitionConstants.UNIT]:
-                                        self._set_unit(j_object)
+                                        self._units = Units(**j_object[DefinitionConstants.UNIT])
                                         is_entity_set[DefinitionConstants.UNIT] = True
                                     else:
                                         self._logger.emit(LOGConstants.ERROR[0], LoggerERROR.print_statements[8] + DefinitionConstants.UNIT)
-                                        exit("Entity Units already set " + self._units)
+                                        exit("Entity Units already set")
 
                                 elif DefinitionConstants.CRITERIA in j_object:
                                     self._set_criteria(j_object)
