@@ -12,8 +12,7 @@ from ..data.data_interface import BinoutData
 
 
 class DEForc(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['x_force', 'y_force', 'z_force', 'displacement', 'resultant_force']
     ids_name = "ids"
 
@@ -32,8 +31,7 @@ class DEForc(BinoutData):
 
 
 class RBDout(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['x_force', 'y_force', 'z_force', 'displacement', 'resultant_force']
     ids_name = "ids"
 
@@ -51,8 +49,7 @@ class RBDout(BinoutData):
 
 
 class BDNout(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['x_force', 'y_force', 'z_force', 'displacement', 'resultant_force']
     ids_name = "ids"
 
@@ -70,6 +67,7 @@ class BDNout(BinoutData):
 
 
 class Disbout(BinoutData):
+    """ """
     DATA_CHANNEL_TYPES = ['r_dis_axial', 'moment_s', 'rslt_nt', 'r_dis_nt', 't_dir_y', 'axial_rot',
                           's_dir_y', 'r_dis_ns', 'matid', 's_dir_x', 'torsion', 'rslt_ns', 'axial_z', 'mtype',
                           't_dir_x', 'rot_s', 'axial_y', 's_dir_z', 'rot_t', 'rslt_axial', 'moment_t', 'axial_x',
@@ -81,6 +79,7 @@ class Disbout(BinoutData):
 
 
 class PartDisbout(BinoutData):
+    """ """
     DATA_CHANNEL_TYPES = ['r_dis_axial', 'moment_s', 'rslt_nt', 'r_dis_nt', 't_dir_y', 'axial_rot',
                           's_dir_y', 'r_dis_ns', 'matid', 's_dir_x', 'torsion', 'rslt_ns', 'axial_z', 'mtype',
                           't_dir_x', 'rot_s', 'axial_y', 's_dir_z', 'rot_t', 'rslt_axial', 'moment_t', 'axial_x',
@@ -100,12 +99,26 @@ class PartDisbout(BinoutData):
         BinoutData.__init__(self, binout, logger, dynasaur_definitions, 'disbout')
 
     def get_channels_ids_object_name(self, object_name, plugin_name):
-        """
-        :return: time array
+        """:return: time array
+
+        Args:
+          object_name: 
+          plugin_name: 
+
+        Returns:
+
         """
         return self._dynasaur_definitions.get_ids_from_name(object_name, "disbout_part", plugin_name)
 
     def get_element_ids_from_part_ids(self, part_ids):
+        """
+
+        Args:
+          part_ids: 
+
+        Returns:
+
+        """
         part_to_element_mapping = self._binout.read(self._name, "matid")
         assert len(part_to_element_mapping.shape) == 2
         d = {}
@@ -117,20 +130,30 @@ class PartDisbout(BinoutData):
 
         return d
 
-    def get_data_of_defined_json(self, json_object, data_offsets):
+    def get_data_of_defined_json(self, data_offsets, type, ID, array, **kwargs):
         """
-        :param: json object
-        :param: data offsets
 
-        :return: data array between data offset and date delta t
+        Args:
+            data_offsets: contains the object type and the first and last time dependent index that should be
+                       considered e.g. [('NODE', 0, 3000)]
+
+            type: dynasaur DataType as defined in utils.constants.DataTypeMapping (OBJECT, NODE, ...)
+
+            ID:   Object Identifier as defined in the object.def file
+
+            array: list of strings, consisting of  the data index in the objects definition file and the channel
+                    type e.g.  ["(0, internal_energy)"]
+
+        Returns:
+          data array between data offset and delta t
+
         """
-        from ..utils.constants import JsonConstants
 
-        part_ids = self.get_channels_ids_object_name(json_object[JsonConstants.ID_UPPER_CASE], self._name)
+        part_ids = self.get_channels_ids_object_name(ID, self._name)
         part_ids_to_element_ids = self.get_element_ids_from_part_ids(part_ids)
 
         assert (len(part_ids_to_element_ids) != 0)
-        array_definition = json_object[JsonConstants.ARRAY]
+        array_definition = array
 
         if array_definition[0].split(',')[0].strip(' (') == "all":
             converted_tuples = [(index, array_definition[0].split(',')[1].strip(' )')) for index, id_ in
@@ -144,7 +167,7 @@ class PartDisbout(BinoutData):
         data_offset = 0
         data_delta_t = -1
         for (t, offset, delta_t) in data_offsets:
-            if t == json_object["type"]:
+            if t == type:
                 data_offset = offset
                 data_delta_t = delta_t
 
@@ -160,6 +183,7 @@ class PartDisbout(BinoutData):
 
 
 class EloutIndex(Enum):
+    """ """
     STRAIN = 0
     STRESS = 1
 
@@ -172,12 +196,13 @@ class EloutIndex(Enum):
 
     @staticmethod
     def translate_element(name):
-        """
-        Translate element with given name
+        """Translate element with given name
 
-        :param: name
+        Args:
+          name: 
+
+        Returns:
 
-        :return EloutIndex(SHELL, SOLID, BEAM):
         """
         name = name.lower()
         if name == "shell":
@@ -189,12 +214,13 @@ class EloutIndex(Enum):
 
     @staticmethod
     def translate_strain_stress(name):
-        """
-        Translate strain or stress with given name
+        """Translate strain or stress with given name
 
-        :param: name
+        Args:
+          name: 
+
+        Returns:
 
-        :return EloutIndex(STRESS, STRAIN):
         """
         name = name.lower()
         if name == "strain":
@@ -204,6 +230,7 @@ class EloutIndex(Enum):
 
 
 class EloutObject(BinoutData):
+    """ """
     DATA_CHANNEL_TYPES = []
     ids_name = "ids"
 
@@ -229,27 +256,58 @@ class EloutObject(BinoutData):
         self._time = None
 
     def get_interpolated_time(self):
-        """
-        :return: no data interpolation for stress and strain types
-        """
+        """:return: no data interpolation for stress and strain types"""
         return self._time
 
-    def get_data_of_defined_json(self, json_object, data_offset):
+    def get_data_of_defined_json(self, data_offsets: list, type: str, ID: str, strain_stress: str,
+                                 strain_stress_type: str = None, **kwargs):
+        """
+        overridden get_data_of_defined_json function.
+
+        Args:
+            data_offsets: contains the object type and the first and last time dependent index that should be
+                       considered e.g. [('OBJECT', 0, 3000)]
+
+            type: dynasaur DataType as defined in utils.constants.DataTypeMapping (OBJECT, NODE, ...)
+
+            ID:   Object Identifier as defined in the object.def file
+
+            strain_stress:  "Strain" or "Stress"
+
+            strain_stress_type: "green_lagrange"
+
+
+
+        Returns:
+            object_data
+
         """
 
-        :param json_object:
-        :param data_offset:
+        assert(self._dynasaur_definitions.is_object_defined(ID))
+        assert(strain_stress in ["Strain", "Stress"])
+        assert(strain_stress_type in [None, "Green_Lagrange"])
 
-        :return dict_data:
-        """
+        strain_stress_idx = EloutIndex.translate_strain_stress(strain_stress)
 
-        strain_stress_idx = EloutIndex.translate_strain_stress(json_object["strain_stress"])
+        part_ids = []
+
+        if not self._dynasaur_definitions.is_object_defined(tmp_object=ID):
+            self._logger.emit(LOGConstants.WARNING[0],
+                              "OBJECT " + ID + " is not defined in your object definition file")
+        else:
+            part_ids = self.get_part_ids_by_object_name(ID, strain_stress_idx)
+            part_ids_defined = self._dynasaur_definitions.get_parts_of_defined_object(ID)
+            if len(part_ids) != len(part_ids_defined):
+                unresolved_part_ids = [part_id_defined for part_id_defined in part_ids_defined if part_id_defined not in part_ids]
+
+                self._logger.emit(LOGConstants.WARNING[0],
+                                  "Unresolved parts in your object definition with ID " + ID + " : " + ", ".join([str(i) for i in unresolved_part_ids])
+                                  + ". Available parts are : " + ", ".join([str(i)for i in self._get_part_ids(strain_stress_idx)]))
 
-        part_ids = self.get_part_ids_by_object_name(json_object["ID"], strain_stress_idx)
         element_count = 0
 
-        for (t, offset, delta_t) in data_offset:
-            if t == json_object["type"]:
+        for (dynasaur_data_type, offset, delta_t) in data_offsets:
+            if dynasaur_data_type == type:
                 data_offset = offset
                 data_delta_t = delta_t
 
@@ -258,10 +316,38 @@ class EloutObject(BinoutData):
         part_data = {}
         part_index = {}
 
-        assert (len(part_ids) > 0)
+        # if len(part_ids) == 0:
+        #     {"part_ids": part_ids, "part_data": part_data,
+        #      "part_idx": part_index, "time_step_indices": time_step_indices,
+        #      "element_type": element_type, "part_value": part_volume,
+        #      "el_by_part_id_el_id": el_by_part_id_el_id,
+        #      "time": self.get_interpolated_time()[time_step_indices]}
+
+
+        #assert (len(part_ids) > 0)
+
+        element_type = None
+        part_volume = None
+        el_by_part_id_el_id = None
 
         for part_id in part_ids:
             index, data = self.get_part_data(strain_stress_idx, part_id, time_step_indices=time_step_indices)
+
+            def log_to_green_lagrange_strain(log_strain):
+                """
+                    Takes the principle logarithmic strains and returns the principle Green-Lagrange strains
+
+                    Args:
+                        log_strain: principle strains (time_steps x object_elems x 3)
+
+                    Returns: Converted Green Lagrange strain
+                """
+
+                return (np.exp(log_strain) ** 2 - 1) / 2
+
+            if strain_stress_type == "Green_Lagrange":
+                data = log_to_green_lagrange_strain(data)
+
             part_data[part_id] = data
             part_index[part_id] = index
 
@@ -273,16 +359,23 @@ class EloutObject(BinoutData):
             part_volume = self.get_part_volume_by_part_ID()
             el_by_part_id_el_id = self.get_element_volume_by_part_id_and_element_id()
 
-        return {"part_ids": part_ids, "part_data": part_data,
-                "part_idx": part_index, "time_step_indices": time_step_indices,
-                "element_type": element_type, "part_value": part_volume,
+        return {"part_ids": part_ids,
+                "part_data": part_data,
+                "part_idx": part_index,
+                "time_step_indices": time_step_indices,
+                "element_type": element_type,
+                "part_value": part_volume,
                 "el_by_part_id_el_id": el_by_part_id_el_id,
                 "time": self.get_interpolated_time()[time_step_indices]}
 
     def _read_volume_data_from_file_volume(self, path_to_volume):
-        """
+        """:return:
+
+        Args:
+          path_to_volume: 
+
+        Returns:
 
-        :return:
         """
         if self._file_data_mapping_element_id_to_volume is not None:
             return True
@@ -308,8 +401,11 @@ class EloutObject(BinoutData):
     def _init_data_volume(self, data_mapping_part_id_to_element_ids):
         """
 
-        :param data_mapping_part_id_to_element_ids:
-        :return:
+        Args:
+          data_mapping_part_id_to_element_ids: return:
+
+        Returns:
+
         """
         if len(self._data_mapping_part_id_to_volume) != 0:
             return True
@@ -334,28 +430,36 @@ class EloutObject(BinoutData):
     def get_part_volume_by_part_ID(self):
         """
 
-        :param part_id:
-        :return:
+        Args:
+          part_id: return:
+
+        Returns:
+
         """
         return self._data_mapping_part_id_to_volume
 
     def get_element_volume_by_part_id_and_element_id(self):
         """
 
-        :param part_id:
-        :param element_id:
-        :return:
+        Args:
+          part_id: param element_id:
+
+        Returns:
+
         """
         return self._data_mapping_part_id_to_element_id_to_volume
 
     def _append_data(self, index_matrix, data_tensor, strain_stress_type, element_type):
         """
 
-        :param index_matrix:
-        :param data_tensor:
-        :param strain_stress_type:
-        :param element_type:
-        :return:
+        Args:
+          index_matrix: param data_tensor:
+          strain_stress_type: param element_type:
+          data_tensor: 
+          element_type: 
+
+        Returns:
+
         """
         if strain_stress_type not in self._data:
             self._data[strain_stress_type] = {}
@@ -373,10 +477,13 @@ class EloutObject(BinoutData):
     def _append_mapping_part_id_to_element_type(self, index_matrix, strain_stress_type, element_type):
         """
 
-        :param index_matrix:
-        :param strain_stress_type:
-        :param element_type:
-        :return:
+        Args:
+          index_matrix: param strain_stress_type:
+          element_type: return:
+          strain_stress_type: 
+
+        Returns:
+
         """
         if strain_stress_type not in self._data_mapping_part_id_to_element_type:
             self._data_mapping_part_id_to_element_type[strain_stress_type] = {}
@@ -393,10 +500,13 @@ class EloutObject(BinoutData):
     def _append_mapping_element_id_to_part_id(self, index_matrix, strain_stress_type, elout_elem):
         """
 
-        :param index_matrix:
-        :param strain_stress_type:
-        :param elout_elem:
-        :return:
+        Args:
+          index_matrix: param strain_stress_type:
+          elout_elem: return:
+          strain_stress_type: 
+
+        Returns:
+
         """
         if strain_stress_type not in self._data_mapping_element_id_to_part_id:
             self._data_mapping_element_id_to_part_id[strain_stress_type] = {}
@@ -420,8 +530,11 @@ class EloutObject(BinoutData):
     def _append_mapping_part_id_to_element_ids(self, strain_stress_type):
         """
 
-        :param strain_stress_type:
-        :return:
+        Args:
+          strain_stress_type: return:
+
+        Returns:
+
         """
         if strain_stress_type not in self._data_mapping_element_id_to_part_id:
             return
@@ -439,8 +552,11 @@ class EloutObject(BinoutData):
     def _concatenated_ranges(self, ranges_list):
         """
 
-        :param ranges_list:
-        :return:
+        Args:
+          ranges_list: return:
+
+        Returns:
+
         """
         ranges_list = np.array(ranges_list, copy=False)
         base_range = np.arange(1, ranges_list.max() + 1)
@@ -452,36 +568,42 @@ class EloutObject(BinoutData):
     def _get_data(self, part_id, strain_stress_type):
         """
 
-        :param part_id:
-        :param strain_stress_type:
-        :return:
+        Args:
+          part_id: param strain_stress_type:
+          strain_stress_type: 
+
+        Returns:
+
         """
         element_type = self._data_mapping_part_id_to_element_type[strain_stress_type][part_id]
         return self._data[strain_stress_type][element_type]
 
     def _get_all_part_ids(self):
-        """
-
-        :return:
-        """
+        """:return:"""
         return np.unique([y for x in [list(self._data_mapping_part_id_to_element_type[vals].keys()) for vals in
                                       self._data_mapping_part_id_to_element_type] for y in x])
 
     def _get_part_ids(self, strain_stress_type):
         """
 
-        :param strain_stress_type:
-        :return:
+        Args:
+          strain_stress_type: return:
+
+        Returns:
+
         """
         return list(self._data_mapping_part_id_to_element_type[strain_stress_type].keys())
 
     def _get_elout_data_and_extend(self, elout_elem, name, extendable_indices):
-        """
-        get the data from binout and extend the given indices with dummy values
-        :param elout_elem:
-        :param name:
-        :param extendable_indices:
-        :return:
+        """get the data from binout and extend the given indices with dummy values
+
+        Args:
+          elout_elem: param name:
+          extendable_indices: return:
+          name: 
+
+        Returns:
+
         """
         if extendable_indices.size == 0:
             return self._binout.read("elout", elout_elem, name).reshape((-1, 1))
@@ -504,9 +626,16 @@ class EloutObject(BinoutData):
         return a[0]
 
     def _info_matrix(self, elout_elem, time, what):
-        """
-            duplicate all entries of first time entry.
+        """duplicate all entries of first time entry.
             part id, integration points, element id
+
+        Args:
+          elout_elem: 
+          time: 
+          what: 
+
+        Returns:
+
         """
         if ((elout_elem == "shell" or elout_elem == "beam") and what == "stress"):
             # ok
@@ -544,23 +673,25 @@ class EloutObject(BinoutData):
         return (np.concatenate((ids, mat, ipts), axis=1)).astype(int)
 
     def _indices_of_destroyed_elements(self, elout_elem, type_):
-        """
-        indices list of destroyed elements over time
+        """indices list of destroyed elements over time
         used to insert dummy values [used for padding to full sized matrix
             -> meaning, each element is available over the entire time]
-
+        
         Functionality:
            in case elements are the destroyed during the simulation
            ids = self._binout.read(self._name, elout_elem, 'ids') has different lengths for each timestep
-
+        
         NOTE:
             if   : stress - shell : extensions for the integration points
         RETURN
             return 1.) insert_index
 
-        :param elout_elem:
-        :param type_:
-        :return:
+        Args:
+          elout_elem: param type_:
+          type_: 
+
+        Returns:
+
         """
 
         array = []
@@ -607,22 +738,30 @@ class EloutObject(BinoutData):
 
         return insert_indices, ind
 
-    def _set_nans(self, w_stress, ind):
-        """
-        set dummy values to NaN (done for deleted elements)
+    def _set_nans(self, w_stress_strain, ind):
+        """set dummy values to NaN (done for deleted elements)
+
+        Args:
+          w_stress_strain: principle components of the strain stress tensor numpy array with shape (n x 3)
+          ind: list of indices, which have to padded with nans
+
+        Returns:
+
         """
         if ind.size != 0:
             self._logger.emit(LOGConstants.WARNING[0], "Evaluated object contains failed elements!")
-            w_stress[ind] = np.tile(np.array([np.NaN]), 3)
+            w_stress_strain[ind] = np.tile(np.array([np.NaN]), 3)
 
     ####################################################################################################################
 
     def _assign_stresses(self, elout_elem):
-        """
-        calculate stress from the elout_elem
+        """calculate stress from the elout_elem
+
+        Args:
+          elout_elem: return:
+
+        Returns:
 
-        :param elout_elem:
-        :return:
         """
 
         if elout_elem != "solid" and elout_elem != "shell" and elout_elem != "beam":
@@ -696,8 +835,11 @@ class EloutObject(BinoutData):
     def _assign_strains(self, elout_elem):
         """
 
-        :param elout_elem:
-        :return:
+        Args:
+          elout_elem: return:
+
+        Returns:
+
         """
         # check the cases where to reject to read strains (if shells or solids do not have an epsilon component)
         self._logger.emit(LOGConstants.READ_BINOUT[0], ("assign strains for " + elout_elem + " elements"))
@@ -793,6 +935,7 @@ class EloutObject(BinoutData):
         self._logger.emit(LOGConstants.READ_BINOUT[0], ("extract data took : " + str(t.time() - start)))
         self._logger.emit(LOGConstants.READ_BINOUT[0], "calc eigenvalue")
         start = t.time()
+
         w_strain = np.linalg.eigvalsh(np.reshape(eps, (-1, 3, 3)))
         self._set_nans(w_strain, ind)
         self._logger.emit(LOGConstants.READ_BINOUT[0], ("calc eigenvalues took : " + str(t.time() - start)))
@@ -813,9 +956,13 @@ class EloutObject(BinoutData):
         self._append_mapping_element_id_to_part_id(index_matrix, EloutIndex.STRAIN, elout_elem)
 
     def read_binout_data(self):
-        """
-        function will be called before a plugin controller is instantiated which needs elout data
+        """function will be called before a plugin controller is instantiated which needs elout data
         :return: True when data is read in, False when no data available
+
+        Args:
+
+        Returns:
+
         """
         # data already read
         if len(self._data) != 0:
@@ -823,14 +970,12 @@ class EloutObject(BinoutData):
 
         # no elout in binout
         if self._name not in self._binout.read():
-            self._logger.emit(LOGConstants.ERROR[0], "no elout data in binout!")
             return False
 
         # actual elout read
         elements = self._binout.read(self._name)
 
         if len(elements) == 0:
-            self._logger.emit(LOGConstants.ERROR[0], 'no elout data available')
             return False
 
         self._logger.emit(LOGConstants.READ_BINOUT[0], 'read elout data ...')
@@ -843,7 +988,7 @@ class EloutObject(BinoutData):
 
         assert (all([time_lengths[0] == length for length in time_lengths]))
 
-        self._time = self._binout.read(self._name, elements[0], 'time')
+        self._time = self._binout.read(self._name, elements[0], 'time') * self._dynasaur_definitions.get_units().time
 
         for elout_elem in elements:
             if elout_elem not in ['shell', 'solid', 'beam'] or len(
@@ -876,33 +1021,39 @@ class EloutObject(BinoutData):
     def get_data_mapping_part_i_dto_element_ids(self, strain_stress_type):
         """
 
-        :param strain_stress_type:
-        :return:
+        Args:
+          strain_stress_type: return:
+
+        Returns:
+
         """
         return self._data_mapping_part_id_to_element_ids[strain_stress_type]
 
     def get_element_type_from_part_id(self, part_id, strain_stress_type):
         """
 
-        :param part_id:
-        :param strain_stress_type:
-        :return:
+        Args:
+          part_id: param strain_stress_type:
+          strain_stress_type: 
+
+        Returns:
+
         """
         return self._data_mapping_part_id_to_element_type[strain_stress_type][part_id]
 
     def get_defined_objects(self):
-        """
-
-        :return:
-        """
+        """:return:"""
         return self._dynasaur_definitions.get_defined_objects_containing_parts(self._get_all_part_ids())
 
     def get_part_ids_by_object_name(self, object_name, strain_stress_type):
         """
 
-        :param object_name:
-        :param strain_stress_type:
-        :return:
+        Args:
+          object_name: param strain_stress_type:
+          strain_stress_type: 
+
+        Returns:
+
         """
         return self._dynasaur_definitions.get_parts_by_object_containing_part_ids(object_name,
                                                                                   self._get_part_ids(
@@ -911,29 +1062,34 @@ class EloutObject(BinoutData):
     def get_element_ids_by_part_id(self, part_id, strain_stress_type):
         """
 
-        :param part_id:
-        :param strain_stress_type:
-        :return:
+        Args:
+          part_id: param strain_stress_type:
+          strain_stress_type: 
+
+        Returns:
+
         """
         return self._data_mapping_part_id_to_element_ids[strain_stress_type][part_id]
 
     def get_part_data(self, strain_stress_type, part_id, element_id=None, time_step_indices=None):
-        """
-        description: return filtered index matrix and data tensor over all time steps for a given part_id
+        """description: return filtered index matrix and data tensor over all time steps for a given part_id
             will return all element, part, integration combinations
-
+        
         optional parameter element_id: filter additional element_id with part_id
             will return just the element part combination with all their integration points
-
+        
         optional parameter time_step_indices: reduce tensor for given time step indices
             will return a reduced data tensor with the given time step indices
             if None: will return all time steps -> full tensor
 
-        :param strain_stress_type:
-        :param part_id:
-        :param element_id:
-        :param time_step_indices:
-        :return:
+        Args:
+          strain_stress_type: param part_id:
+          element_id: param time_step_indices: (Default value = None)
+          part_id: 
+          time_step_indices:  (Default value = None)
+
+        Returns:
+
         """
         tmp_data = self._get_data(part_id, strain_stress_type)
 
@@ -955,9 +1111,12 @@ class EloutObject(BinoutData):
     def get_element_type_name(self, element_id, strain_stress_type):
         """
 
-        :param element_id:
-        :param strain_stress_type:
-        :return:
+        Args:
+          element_id: param strain_stress_type:
+          strain_stress_type: 
+
+        Returns:
+
         """
         element_types = []
         for element_type in self._data_mapping_element_id_to_part_id[strain_stress_type].keys():
@@ -967,35 +1126,40 @@ class EloutObject(BinoutData):
         return element_types
 
     def get_integration_point_indices_of_part_data_by_element_id(self, index, element_id):
-        """
-        further filtering by element_id of given results from getPartData()
-
+        """further filtering by element_id of given results from getPartData()
+        
         index and data are return values of getPartData()
-
+        
         return row indices of filtered elements
          - returns list with one indices when just one integration point is available
          - returns list with multiple indices when multiple integration points are available
 
-        :param index:
-        :param element_id:
-        :return:
+        Args:
+          index: param element_id:
+          element_id: 
+
+        Returns:
+
         """
         return np.where(index[:, 0] == element_id)[0]
 
     def get_element_ids_of_part_data(self, index):
-        """
-        parameter uses index matrix of getPartData()
+        """parameter uses index matrix of getPartData()
         using this function instead of the data_mappings will enhance performance
-
+        
         get element IDs of filtered getPartData()
 
-        :param index:
-        :return:
+        Args:
+          index: return:
+
+        Returns:
+
         """
         return np.unique(index[:, 0])
 
 
 class Elout(BinoutData):
+    """ """
     # applies for beam elements!
     DATA_CHANNEL_TYPES = ['axial', 'shear_s', 'shear_t', 'moment_s', 'moment_t', 'torsion']
     ids_name = "ids"
@@ -1018,7 +1182,7 @@ class Elout(BinoutData):
         :param elem_name:
         :return:
         """
-        self._time = self._binout.read(*subdatatypes, 'time').flatten()
+        self._time = self._binout.read(*subdatatypes, 'time').flatten() * self._dynasaur_definitions.get_units().time
         self._time_interp = self._interp_time(self._time)
         self._ids = np.array(self._binout.read(*subdatatypes, self.__class__.ids_name)[0])
         try:
@@ -1031,14 +1195,18 @@ class Elout(BinoutData):
     def _get_subdatatypes_to_reach_binout_data(self, ls, ls_subdatatypes):
         """
 
-        :param ls: 
-        :param ls_subdatatypes: 
-        :return: 
+        Args:
+          ls: param ls_subdatatypes:
+          ls_subdatatypes: 
+
+        Returns:
+
         """
         ls_subdatatypes.append([self._name, 'beam'])
 
 
 class Abstat(BinoutData):
+    """ """
     # applies for beam elements!
     DATA_CHANNEL_TYPES = ['dm_dt_out', 'pressure', 'internal_energy', 'density', 'surface_area', 'dm_dt_in', 'dm_dt_outp',
                           'reaction', 'volume', 'gas_temp', 'total_mass', 'dm_dt_outv']
@@ -1059,6 +1227,7 @@ class Abstat(BinoutData):
 
 
 class AbstatCPM(BinoutData):
+    """ """
     # applies for beam elements!
     DATA_CHANNEL_TYPES = ['dm_dt_out', 'pressure', 'internal_energy', 'density', 'surface_area', 'dm_dt_in',
                           'inflator_e', 'reaction', 'volume', 'gas_temp', 'total_mass', 'pres_particle', 'Trans_ke']
@@ -1079,14 +1248,14 @@ class AbstatCPM(BinoutData):
 
 
 class Glstat(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['global_y_velocity', 'internal_energy', 'kinetic_energy', 'percent_increase', 'ts_element',
                           'energy_ratio_wo_eroded', 'spring_and_damper_energy', 'eroded_kinetic_energy',
                           'num_bad_shells', 'energy_ratio', 'external_work', 'time_step', 'global_z_velocity',
                           'global_x_velocity', 'eroded_internal_energy', 'time', 'total_energy',
                           'system_damping_energy', 'hourglass_energy', 'sliding_interface_energy', 'cycle', 'ts_eltype',
-                          'joint_internal_energy', 'eroded_hourglass_energy', 'nzc', 'added_mass','resultant_global_velocity','ts_part']
+                          'joint_internal_energy', 'eroded_hourglass_energy', 'nzc', 'added_mass',
+                          'resultant_global_velocity', 'ts_part']
 
     ids_name = "ids"
 
@@ -1105,24 +1274,32 @@ class Glstat(BinoutData):
     def _get_subdatatypes_to_reach_binout_data(self, ls, ls_subdatatypes):
         """
 
-        :param ls:
-        :param ls_subdatatypes:
-        :return:
+        Args:
+          ls: param ls_subdatatypes:
+          ls_subdatatypes: 
+
+        Returns:
+
         """
         ls_subdatatypes.append([ls[0]])
 
     def _read_data_channels(self, subdatatypes=[]):
-        """
-        initialisation of self._data
-
+        """initialisation of self._data
+        
         function reads available binout data and interpolates it to a targeted interpolation time
         creates subset of available channels (in binout) and defined DATA_CHANNEL_TYPES for Glstat
         INFO: own implementation in glstat.py due to missing ids for glstat data
+
+        Args:
+          subdatatypes:  (Default value = [])
+
+        Returns:
+
         """
         data = {}
         available_channels = [value for value in self.__class__.DATA_CHANNEL_TYPES if
                               value in self._binout.read(self._name)]
-
+        self._ids = np.array([0])
         for key in available_channels:
             type_data = self._binout.read(self._name, key)
 
@@ -1139,10 +1316,19 @@ class Glstat(BinoutData):
         self._data = {**self._data, **data}
 
     def get_measurement_channel(self, id_, channel_name):
-        """
-        returns interpolated data of the channel name from self._data
+        """returns interpolated data of the channel name from self._data
         identified by channel_name
+
+        Args:
+          id_: 
+          channel_name: 
+
+        Returns:
+
         """
+
+        unit = self._get_unit_dimension_from_channel_name(channel_name)
+
         if channel_name == 'time':
             return self._time_interp.reshape(-1, 1)
 
@@ -1152,12 +1338,11 @@ class Glstat(BinoutData):
         self._logger.emit(LOGConstants.DATA_PLUGIN[0], 'ENERGY_GLOBAL read from channel name: ' + channel_name)
 
         d = self._data[channel_name]
-        return d
+        return d * unit
 
 
 class Matsum(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['x_momentum', 'y_rbvelocity', 'z_rbvelocity', 'internal_energy', 'kinetic_energy',
                           'max_shell_mass', 'y_momentum', 'mass', 'eroded_kinetic_energy', 'brick_id', 'max_brick_mass',
                           'shell_id', 'eroded_internal_energy', 'z_momentum', 'hourglass_energy',
@@ -1183,8 +1368,7 @@ class Matsum(BinoutData):
 
 
 class RigidBody(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['local_ax', 'local_ay', 'local_az', 'local_dx', 'local_dy', 'local_dz', 'local_vx',
                           'local_vy', 'local_vz', 'local_rax', 'local_ray', 'local_raz', 'local_rdx', 'local_rdy',
                           'local_rdz', 'local_rvx', 'local_rvy', 'local_rvz', 'global_x', 'global_y', 'global_z',
@@ -1211,8 +1395,7 @@ class RigidBody(BinoutData):
 
 
 class BoundaryCondition(BinoutData):
-    """
-    """
+    """ """
     # DATA_CHANNEL_TYPES = ['ids', 'energy', 'revision', 'y_force', 'zmoment', 'x_force', 'title', 'y_total',
     #                       'legend', 'version', 'xmoment', 'ymoment', 'legend_ids', 'z_force', 'date', 'time',
     #                       'etotal', 'x_total', 'z_total']
@@ -1237,8 +1420,7 @@ class BoundaryCondition(BinoutData):
 
 
 class Nodout(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['rx_velocity', 'ry_velocity', 'rz_velocity', 'rx_displacement', 'ry_displacement',
                           'rz_displacement', 'rx_acceleration', 'ry_acceleration', 'rz_acceleration', 'x_coordinate',
                           'y_coordinate', 'z_coordinate', 'x_displacement', 'y_displacement', 'z_displacement',
@@ -1264,10 +1446,9 @@ class Nodout(BinoutData):
 
 
 class JointForc(BinoutData):
-    """
-    """
-    DATA_CHANNEL_TYPES = [ 'energy', 'resultant_moment', 'resultant_force', 'x_force',  'y_force',
-                           'z_force', 'x_moment', 'y_moment', 'z_moment']
+    """ """
+    DATA_CHANNEL_TYPES = ['energy', 'resultant_moment', 'resultant_force', 'x_force',  'y_force', 'z_force',
+                          'x_moment', 'y_moment', 'z_moment']
     ids_name = "ids"
 
     def __init__(self, data, logger, dynasaur_definitions):
@@ -1287,11 +1468,17 @@ class JointForc(BinoutData):
     def get_measurement_channel(self, id_, channel_name):
         """
 
-        :param id:
-        :param channel_name:
-        :return:
+        Args:
+          id: param channel_name:
+          id_: 
+          channel_name: 
+
+        Returns:
+
         """
 
+        unit = self._get_unit_dimension_from_channel_name(channel_name)
+
         if channel_name == 'time':
             return self._time_interp.reshape(-1, 1)
 
@@ -1303,7 +1490,7 @@ class JointForc(BinoutData):
 
         if len(data_index) == 0:
             self._logger.emit(LOGConstants.ERROR[0], 'ID ' + str(id_) + ' not in binout')
-            exit()
+            return []
 
         self._logger.emit(LOGConstants.DATA_PLUGIN[0], 'read id ' + str(id_) + ' from channel name: ' + channel_name)
         assert (len(data_index) >= 1)
@@ -1314,30 +1501,11 @@ class JointForc(BinoutData):
 
         d = self._data[channel_name][:, data_index]
 
-        return d.reshape(-1, len(data_index))
-
-        if channel_name == 'time':
-            return self._time.reshape(-1, 1)
-
-
-
-        if len(data_index) == 0:
-            self._logger.emit(LOGConstants.ERROR[0], 'ID ' + str(id_nr) + ' not in binout')
-            exit()
-        data_index = [data_index[id_slave_master]]
-
-        self._logger.emit(LOGConstants.DATA_PLUGIN[0], 'read id ' + str(id_) + ' from channel name: ' + channel_name)
-        assert (len(data_index) >= 1)
-
-        if channel_name not in self._data.keys():
-            self._logger.emit(LOGConstants.ERROR[0], str(id_) + ' has no data with the identifier : ' + channel_name)
-            return []
-
-        d = self._data[channel_name][:, data_index]
-        return d.reshape(-1, len(data_index))
+        return d.reshape(-1, len(data_index)) * unit
 
 
 class RCForc(BinoutData):
+    """ """
     DATA_CHANNEL_TYPES = ['x_moment', 'y_moment', 'z_moment', 'x_force', 'y_force', 'z_force', 'tie_count', 'mass',
                           'tie_area', 'resultant_force']
 
@@ -1358,9 +1526,13 @@ class RCForc(BinoutData):
     def get_measurement_channel(self, id_, channel_name):
         """
 
-        :param id:
-        :param channel_name:
-        :return:
+        Args:
+          id: param channel_name:
+          id_: 
+          channel_name: 
+
+        Returns:
+
         """
 
         if not isinstance(self._binout, lasso.dyna.Binout):
@@ -1369,6 +1541,8 @@ class RCForc(BinoutData):
             # maybe changed
             return BinoutData.get_measurement_channel(self, id_, channel_name)
 
+        unit = self._get_unit_dimension_from_channel_name(channel_name)
+
         if channel_name == 'time':
             return self._time.reshape(-1, 1)
 
@@ -1383,7 +1557,7 @@ class RCForc(BinoutData):
 
         if len(data_index) == 0:
             self._logger.emit(LOGConstants.ERROR[0], 'ID ' + str(id_nr) + ' not in binout')
-            exit()
+            return []
         data_index = [data_index[id_slave_master]]
 
         self._logger.emit(LOGConstants.DATA_PLUGIN[0], 'read id ' + str(id_) + ' from channel name: ' + channel_name)
@@ -1394,12 +1568,11 @@ class RCForc(BinoutData):
             return []
 
         d = self._data[channel_name][:, data_index]
-        return d.reshape(-1, len(data_index))
+        return d.reshape(-1, len(data_index)) * unit
 
 
 class SBTout(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['belt_length', 'belt_force']
     ids_name = "belt_ids"
 
@@ -1417,8 +1590,7 @@ class SBTout(BinoutData):
 
 
 class Secforc(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['x_force', 'y_force', 'z_force', 'x_moment', 'y_moment', 'z_moment',
                           'x_centroid', 'y_centroid', 'z_centroid', 'total_force', 'total_moment', 'area']
     # 'total_force', 'total_moment',  'area'
@@ -1442,8 +1614,7 @@ class Secforc(BinoutData):
 
 
 class Sleout(BinoutData):
-    """
-    """
+    """ """
     DATA_CHANNEL_TYPES = ['total_friction', 'slave', 'total_master', 'master', 'cycle',
                           'friction_energy', 'time', 'total_energy',
                           'total_slave']
@@ -1463,14 +1634,22 @@ class Sleout(BinoutData):
         BinoutData.__init__(self, binout, logger, dynasaur_definitions, 'sleout')
 
     def get_measurement_channel(self, id_, channel_name):
-        """
-        returns the interpolated channel,
+        """returns the interpolated channel,
         Aim: Constant time interval
-        :param id:
-        :param channel_name:
-        :return:
+
+        Args:
+          id: param channel_name:
+          id_:
+          channel_name:
+
+        Returns:
+
+
         """
+
+        unit = self._get_unit_dimension_from_channel_name(channel_name)
+
         if channel_name == 'time':
             return self._time.reshape(-1, 1)
-        d = self._data[channel_name][:, 0]
+        d = self._data[channel_name][:, 0] * unit
         return d
