@@ -2,13 +2,15 @@ import abc
 import inspect
 import time
 import numpy as np
-import h5py
+import csv
 from datetime import datetime
+
 from lasso.dyna.Binout import Binout
 
-from ..utils.constants import StandardFunctionsDefinition, JsonConstants, ObjectConstantsForData, LoggerERROR
+from ..utils.constants import StandardFunctionsDefinition, JsonConstants, ObjectConstantsForData, \
+    LoggerERROR, LoggerWARNING
 from ..calc.standard_functions import StandardFunction
-from ..utils.constants import LOGConstants, PluginsParamDictDef
+from ..utils.constants import LOGConstants
 from ..utils.logger import ConsoleLogger
 from ..data.dynasaur_definitions import DynasaurDefinitions
 from ..data.data_container import DataContainer
@@ -17,26 +19,29 @@ from ..data.madymo import MadymoData
 
 
 class Plugin(object):
-    def __init__(self, path_to_def_file, path_def_file_id, data_source, name, user_function_object=None,
+    """ """
+    def __init__(self, calculation_procedure_def_file, object_def_file, data_source, name, user_function_object=None,
                  volume_path=None, code_type="LS-DYNA"):
         """
-        Initialization class constructor
+        Plugin constructor
 
-        :param: calculation_procedure_def_file
-        :param: object_def_file
-        :param: data_source(binout path)
-        :param: name
-        :param: logger
-        :param: dynasaur definition
+        Args:
+            calculation_procedure_def_file: path to calculation_procedure file
+            object_def_file: path to object_def file
+            data_source: path to binout
+            user_function_object:
+            code_type: [LS-DYNA, VPS, MADYMO]
+            dynasaur definition
 
-        :return:
+        Returns:
+            Instance of the DataVisualizationController
         """
+
         self._logger = ConsoleLogger()
         self._dynasaur_definitions = DynasaurDefinitions(self._logger)
-        self._dynasaur_definitions.read_def(path_to_def_file)
-        self._dynasaur_definitions.read_def(path_def_file_id)
+        self._dynasaur_definitions.read_def(calculation_procedure_def_file)
+        self._dynasaur_definitions.read_def(object_def_file)
         self._user_function = user_function_object
-        # self._code_type = self._dynasaur_definitions.get_code()
 
         if code_type == "LS-DYNA":
             binout = Binout(data_source)
@@ -52,8 +57,7 @@ class Plugin(object):
             DataContainer.init_all_data_sources(madymo_data, self._logger, self._dynasaur_definitions,
                                                 volume_path=volume_path)
         else:
-            # TODO: Error msg
-            print("Not valid code type")
+            self._logger.emit(LOGConstants.ERROR[0], "invalid code type try, [LS-DYNA, MADYMO, VPS]")
             return
 
         self._timestamp = datetime.today().strftime("%Y-%m-%d-%H-%M-%S")
@@ -66,11 +70,14 @@ class Plugin(object):
         # ensure unique timestamps
         time.sleep(1)
 
-
     def _check_if_user_function(self, function_name):
         """
-        :param function_name:
-        :return bool:
+        check if a function is defined as user_function
+
+        Args:
+          function_name:
+
+        Returns: True or False
         """
         attributes = inspect.getmembers(self._user_function, predicate=inspect.isfunction)
         if function_name in [att[0] for att in attributes]:
@@ -81,22 +88,28 @@ class Plugin(object):
     def _kwargs_from_param_dict(param_dict, units):
         """
 
-        :param param_dict:
-        :return:
+        Args:
+          param_dict: return:
+          units: 
+
+        Returns:
+
         """
         kwargs = {}
         for key in param_dict:
             kwargs[key] = param_dict[key]
-
-        kwargs["units"] = units
+        #kwargs["units"] = units
         return kwargs
 
     def _call_standard_function(self, function_name, param_dict):
         """
 
-        :param function_name:
-        :param param_dict:
-        :return:
+        Args:
+          function_name: param param_dict:
+          param_dict: 
+
+        Returns:
+
         """
         kwargs = Plugin._kwargs_from_param_dict(param_dict, self._units)
         if self._user_function is not None:
@@ -112,14 +125,21 @@ class Plugin(object):
             return None
         else:
             func_ptr = StandardFunction.__dict__[function_name]
-            return func_ptr.__func__(**kwargs)
+            try:
+                ret = func_ptr.__func__(**kwargs)
+            except (AttributeError, np.AxisError):
+                ret = None
+            return ret
 
     def _get_data_from_dynasaur_json(self, json_object, data_offsets):
         """
 
-        :param json_object:
-        :param data_offsets:
-        :return:
+        Args:
+          json_object: param data_offsets:
+          data_offsets: 
+
+        Returns:
+
         """
         if JsonConstants.FUNCTION in json_object.keys():  # expected name and params
             function_name = json_object[JsonConstants.FUNCTION][JsonConstants.NAME]
@@ -141,11 +161,11 @@ class Plugin(object):
                 self._logger.emit(LOGConstants.ERROR[0], "Calculation of " + function_name + " aborted!")
 
         elif JsonConstants.VALUE in json_object.keys():
-            return np.array(json_object[JsonConstants.VALUE])
+            return np.array(json_object[JsonConstants.VALUE]) * self._dynasaur_definitions.get_units().dimensionless
 
         elif JsonConstants.TYPE in json_object.keys():
             data_type = json_object[JsonConstants.TYPE]
-            return self._data[data_type].get_data_of_defined_json(json_object, data_offsets)
+            return self._data[data_type].get_data_of_defined_json(data_offsets, **json_object)
 
         else:
             assert False
@@ -153,25 +173,33 @@ class Plugin(object):
     def init_plugin_data(self, update):
         """
 
-        :param update:
-        :return:
+        Args:
+          update: return:
+
+        Returns:
+
         """
         if self._data is None or update:
             self._data = {data: DataContainer.get_data(data) for data in self._data_requirements}
             for data_name in self._data.keys():
                 if self._data[data_name] is None:
                     self._logger.emit(LOGConstants.ERROR[0], data_name + LoggerERROR.print_statements[1] +
-                                        ", ".join(sorted([i for i in ObjectConstantsForData.__dict__.keys() if i[:1] != '_'])))
+                                      ', '.join(sorted([i for i in ObjectConstantsForData.__dict__.keys()
+                                                        if i[:1] != '_'])))
                 if not self._data[data_name].read_binout_data():
-                    self._logger.emit(LOGConstants.ERROR[0], data_name + LoggerERROR.print_statements[2])
+                    self._logger.emit(LOGConstants.WARNING[0], data_name + LoggerWARNING.print_statements[3])
 
     def _reduce_sample_offset(self, json_object, sample_offsets):
         """
 
-        :param json_object:
-        :param data_offsets:
-
+        Args:
+          json_object: param data_offsets:
+        
         :return sample:
+          sample_offsets: 
+
+        Returns:
+
         """
         self._get_data_from_json_reduce_samples(json_object)
         self._sample_types = list(set(self._sample_types))
@@ -182,23 +210,19 @@ class Plugin(object):
         self._sample_types.clear()
         return return_sample
 
-    def _get_sample_offset(self, param_dict):
-        """
-        get sample offset defined between start and end time
+    def _get_sample_offset(self, dynasaur_json, t_start, t_end):
+        """get sample offset defined between start and end time
+
+        Args:
+          param_dict: return sample offset:
+
+        Returns:
 
-        :param param_dict:
-        :return sample offset:
         """
-        start_time = param_dict[PluginsParamDictDef.START_TIME] if PluginsParamDictDef.START_TIME in param_dict \
-                                                                   and param_dict[
-                                                                       PluginsParamDictDef.START_TIME] != "None" else None
-        end_time = param_dict[PluginsParamDictDef.END_TIME] if PluginsParamDictDef.END_TIME in param_dict \
-                                                               and param_dict[
-                                                                   PluginsParamDictDef.END_TIME] != "None" else None
 
         sample_offsets = []
         data_types_in_json = []
-        self._dynasaur_definitions.get_all_data_types_from_json(d=param_dict["dynasaur_json"], ls=data_types_in_json)
+        self._dynasaur_definitions.get_all_data_types_from_json(d=dynasaur_json, ls=data_types_in_json)
         for key in self._data.keys():
             if key not in data_types_in_json:
                 continue
@@ -209,25 +233,27 @@ class Plugin(object):
             if self._data[key].get_interpolated_time() is None:
                 return
 
-            if start_time is not None and end_time is not None:
+            if t_start is not None and t_end is not None:
                 indices = np.argwhere(
-                    (start_time * self._units.second() <= self._data[key].get_interpolated_time()) &
-                    (self._data[key].get_interpolated_time() <= end_time * self._units.second()))
+                    (t_start * self._units.time <= self._data[key].get_interpolated_time()) &
+                    (self._data[key].get_interpolated_time() <= t_end * self._units.time))
                 if len(indices) == 0:
                     self._logger.emit(LOGConstants.ERROR[0],
-                                      "Check t_start, which was set to " + str(start_time * self._units.second()))
+                                      "Check t_start, which was set to " + str(t_start * self._units.time))
                     return
-                if start_time >= end_time:
-                    end_time = self._data[key].get_()[-1]
+                if t_start >= t_end:
+                    t_end = self._data[key].get_()[-1]
                     self._logger.emit(LOGConstants.ERROR[0],
-                                      "End time is smaller or equal as start time! End time set to " + str(end_time))
+                                      "End time is smaller or equal as start time! End time set to " + str(t_end))
                     return
 
-            elif start_time is not None and end_time is None:
-                indices = np.argwhere(start_time * self._units.second() <= self._data[key].get_interpolated_time())
+            elif t_start is not None and t_end is None:
+                indices = np.argwhere(t_start * self._units.time <=
+                                      self._data[key].get_interpolated_time())
 
-            elif start_time is None and end_time is not None:
-                indices = np.argwhere(self._data[key].get_interpolated_time() <= end_time * self._units.second())
+            elif t_start is None and t_end is not None:
+                indices = np.argwhere(self._data[key].get_interpolated_time() <=
+                                      t_end * self._units.time)
 
             else:
                 indices = np.array([np.arange(0, len(self._data[key].get_interpolated_time()))]).transpose()
@@ -237,12 +263,13 @@ class Plugin(object):
         return sample_offsets
 
     def _get_data_from_json_reduce_samples(self, json_object):
-        """
-        get sample offset defined between start and end time
+        """get sample offset defined between start and end time
 
-        :param json_object:
+        Args:
+          json_object: return sample offset:
+
+        Returns:
 
-        :return sample offset:
         """
         if JsonConstants.FUNCTION in json_object.keys():  # expected name and params
             parameter_def = json_object[JsonConstants.FUNCTION][JsonConstants.PARAM]
@@ -255,7 +282,7 @@ class Plugin(object):
                     param_dict[key] = parameter_def[key]
         else:  # data to obtain
             if JsonConstants.VALUE in json_object.keys():
-                return np.array(json_object[JsonConstants.VALUE])
+                return np.array(json_object[JsonConstants.VALUE]) * self._dynasaur_definitions.get_units().dimensionless
             elif "strain_stress" in json_object.keys():
                 self._sample_types.append(json_object[JsonConstants.TYPE])
                 return json_object["strain_stress"]
@@ -265,50 +292,82 @@ class Plugin(object):
             else:
                 assert False
 
+    def _write_header_and_content_to_csv(self, path, header_info, values):
+        """
+
+        :return:
+        """
+        with open(path, 'w') as csv_file:
+            writer = csv.writer(csv_file, delimiter=';', lineterminator='\n')
+            t_m = list(zip(*header_info))
+            for key in t_m:
+                writer.writerow(key)
+            t_m = list(zip(*values))
+            for key in t_m:
+                writer.writerow(key)
+
 
 class PluginInterface(metaclass=abc.ABCMeta):
+    """ """
 
     @abc.abstractmethod
-    def _calculate_and_store_results(self, param_dict, to_si):
+    def _calculate_and_store_results(self, *args, **kwargs):
         """
-        :param param_dict:
-        :return:
+
+        Args:
+          param_dict: return:
+
+        Returns:
+
         """
 
     @abc.abstractmethod
     def get_defined_calculation_procedures(self):
         """
 
-        :return:
+        Args:
+
+        Returns:
+
         """
 
-    def calculate(self, param_dict, to_si=False):
+    def calculate(self, param_dict):
         """
 
-        :param param_dict:
-        :param to_si
-        :return:
+        Args:
+          param_dict: param to_si
+          to_si:  (Default value = False)
+
+        Returns:
+
         """
         param_dict_data = self._dynasaur_definitions.get_param_dict_from_command(command=param_dict)
-        self._calculate_and_store_results(param_dict_data, to_si=to_si)
+        self._calculate_and_store_results(**param_dict_data)
 
     def get_object_data(self, type_, id_, strain_stress, index, channel, data_offset):
         """
 
-        :param type_:
-        :param id:
-        :param strain_stress:
-        :param data_offset:
-        :return:
+        Args:
+          type_: param id:
+          strain_stress: param data_offset:
+          id_: 
+          index: 
+          channel: 
+          data_offset: 
+
+        Returns:
+
         """
         # print(self._data)
         # part_ids = self._data[type_]._dynasaur_definitions._objects[id]._parts.keys()
         if type_ == "OBJECT":
-            return self._data[type_].get_data_of_defined_json(
-                json_object={"type": type_, "ID": id_, "strain_stress": strain_stress},
-                data_offset=[(type_, data_offset[0], data_offset[1])])
+            return self._data[type_].get_data_of_defined_json(data_offsets=[(type_, data_offset[0], data_offset[1])],
+                                                              type=type_,
+                                                              ID=id_,
+                                                              strain_stress=strain_stress)
 
         else:
-            return self._data[type_].get_data_of_defined_json(
-                json_object={"type": type_, "ID": id_, "array": ["(" + str(index) + "," + str(channel) + ")"]},
-                data_offsets=[(type_, data_offset[0], data_offset[1])])
+            return self._data[type_].get_data_of_defined_json(data_offsets=[(type_, data_offset[0], data_offset[1])],
+                                                              type=type_,
+                                                              ID=id_,
+                                                              array=["(" + str(index) + "," + str(channel) + ")"])
