@@ -3,11 +3,13 @@ import datetime
 
 
 class DefFileConstants(object):
+    """ """
     FUNC_CRITERIA = "criteria"
     FUNC_DATA_VISUALIZATION = "data_visualization"
 
 
 class IsommeConstants(object):
+    """ """
     CHANNELS = "Channels"
     CHANNEL = "Channel"
     INSTUMENTATION_STANDARD = "Not applicable"
@@ -71,6 +73,7 @@ class IsommeConstants(object):
 
 
 class TransducerConstants(object):
+    """ """
     COORDINATE = "Coordinate"
     FORCE = "Force"
     TIME = "Time"
@@ -81,6 +84,7 @@ class TransducerConstants(object):
 
 
 class UnitsConstants(object):
+    """ """
     TIME = "time"
     LENGTH = "length"
     WEIGHT = "weight"
@@ -99,6 +103,7 @@ class UnitsConstants(object):
 
 
 class MadymoConstants:
+    """ """
     CHANNEL_NAME = "channel_name"
     COMP = "COMP"
     Y_VALUES = "Y_VALUES"
@@ -108,6 +113,7 @@ class MadymoConstants:
 
 
 class DefinitionConstants(object):
+    """ """
     TITLE = "TITLE"
     UNIT = "UNIT"
     OBJECTS = "OBJECTS"
@@ -117,6 +123,7 @@ class DefinitionConstants(object):
 
 
 class TestConstants:
+    """ """
     ELOUT = "_element"
     DATA_VIS = "_data_vis"
     INJURY_CRIT = "_injury_crit"
@@ -132,6 +139,7 @@ class TestConstants:
 
 
 class ObjectConstantsForData:
+    """ """
     ELEMENT = "ELEMENT"
     AIRBAG = "AIRBAG"
     AIRBAG_CPM = "AIRBAG_CPM"
@@ -152,6 +160,7 @@ class ObjectConstantsForData:
 
 
 class DataTypeMapping:
+    """ """
     mapping = [(ObjectConstantsForData.ELEMENT, "elout"),
                 (ObjectConstantsForData.AIRBAG, "abstat"),
                 (ObjectConstantsForData.AIRBAG_CPM, "abstat_cpm"),
@@ -174,8 +183,11 @@ class DataTypeMapping:
     def binary_identifier_2_dynasaur_identifier(binary_data_type):
         """
 
-        :param binary_data_type:
-        :return:
+        Args:
+          binary_data_type: return:
+
+        Returns:
+
         """
 
         for tpl in DataTypeMapping.mapping:
@@ -189,8 +201,11 @@ class DataTypeMapping:
     def dynasaur_identifier_2_binary_identifier(dynasaur_data_type):
         """
 
-        :param dynasaur_data_type:
-        :return:
+        Args:
+          dynasaur_data_type: return:
+
+        Returns:
+
         """
         for tpl in DataTypeMapping.mapping:
             if tpl[0] == dynasaur_data_type:
@@ -201,6 +216,7 @@ class DataTypeMapping:
 
 
 class JsonConstants:
+    """ """
     TYPE = "type"
     ID = "id"
     ID_UPPER_CASE = "ID"
@@ -228,6 +244,7 @@ class JsonConstants:
 
 
 class PluginsParamDictDef:
+    """ """
     DYNASAUR_JSON = "dynasaur_json"
     START_TIME = "t_start"
     END_TIME = "t_end"
@@ -236,20 +253,22 @@ class PluginsParamDictDef:
     AGE = 'age'
     X_LABEL = "x_label"
     Y_LABEL = "y_label"
+    X_UNIT = "x_unit"
+    Y_UNIT = "y_unit"
     SELECTION_TENSION_COMPRESSION = "selection_tension_compression"
     INTEGRATION_POINT = "integration_point"
     PERCENTILE = "percentile"
 
 
 class OutputStringForPlugins:
+    """ """
     VALUE = 'Value'
     LIMITS = "Limits"
-    ALL_MAX = "allmax"
-    FRAC_MAX = "fracmax"
-    ALL_FRAC = "allfrac"
+    UNIT = "unit"
 
 
 class DataPluginConstants:
+    """ """
     TIME = 'time'
     VISUALIZATION = "visualization"
     X = "x"
@@ -257,6 +276,7 @@ class DataPluginConstants:
 
 
 class StandardFunctionsDefinition:
+    """ """
     HIC15 = "HIC_15"
     HIC36 = "HIC_36"
     ERROR_NOT_STANDARD_FUNCTION = " is not a standard or user function"
@@ -264,6 +284,7 @@ class StandardFunctionsDefinition:
 
 
 class LOGConstants(object):
+    """ """
     ERROR = ("[ERROR]", "red")
     WARNING = ("[WARNING]", "magenta")
     READ_BINOUT = ("[READ BINOUT]", "blue")
@@ -276,6 +297,7 @@ class LOGConstants(object):
 
 
 class LoggerDefinitionFileValidator:
+    """ """
     ID_MUST_BE_DEFINITED = "ID has to be defied!"
     ID_IS_INTEGER_LIST = "ID has to be a list of integers!"
     DEFINITION_MISSING = "Definition is missing!"
@@ -302,9 +324,9 @@ class LoggerDefinitionFileValidator:
 
 
 class LoggerERROR:
+    """ """
     print_statements = {
         1: " is not supported as data type. Use ",
-        2: " not available in your binout",
         3: " No file to select",
         4: " JSON object issue: %s",
         5: " Definition does not contain key ",
@@ -315,14 +337,18 @@ class LoggerERROR:
 
 
 class LoggerWARNING:
+    """ """
     print_statements = {
-        1: " is not a valid definition!",
+        1: " is not a valid DYNASAUR type! Available types are: ",
         2: " DATA VISUALIZATION plugin: Definition does not contain header information for the generating ISO-MME files."
-           " The information is preset to [" + "NOVALUE , " + datetime.date.today().strftime("%d/%m/%Y") + ", 1, 6yo]"
+           " The information is preset to [" + "NOVALUE , " + datetime.date.today().strftime("%d/%m/%Y") + ", 1, 6yo]",
+        3: " used in your calculation procedure file, but not available in your binout"
+
     }
 
 
 class LoggerSCRIPT:
+    """ """
     print_statements = {
         1: " writing csv to ",
         2: " writing pdf to ",
@@ -334,11 +360,13 @@ class LoggerSCRIPT:
 
 
 class LoggerReadDynasaurDefinitionFile:
+    """ """
     READ = "read %s"
     DONE = "done"
 
 
 class VPSDataConstant:
+    """ """
     NODOUT = "nodout"
     NODE = "NODE"
     SECFORC = "secforc"
@@ -373,6 +401,7 @@ class VPSDataConstant:
     ENERGY_GLOBAL_ENKIT = "ENKIT"
 
 class DataChannelTypesNodout:
+    """ """
     X_COORDINATE = "x_coordinate"
     Y_COORDINATE = "y_coordinate"
     Z_COORDINATE = "z_coordinate"
@@ -403,11 +432,14 @@ class DataChannelTypesNodout:
 
 
 class DataChannelTypesContact:
+    """ """
     X_FORCE = "x_force"
     Y_FORCE = "y_force"
     Z_FORCE = "z_force"
+    TIE_AREA = "tie_area"
 
 class DataChannelTypesGlobalEnergy:
+    """ """
     EXTERNAL_WORK = "external_work"
     INTERNAL_ENERGY = "internal_energy"
     KINETIC_ENERGY = "kinetic_energy"
@@ -415,6 +447,7 @@ class DataChannelTypesGlobalEnergy:
 
 
 class DataChannelTypesSecforc:
+    """ """
     X_CENTROID = "x_centroid"
     Y_CENTROID = "y_centroid"
     Z_CENTROID = "z_centroid"
@@ -428,3 +461,8 @@ class DataChannelTypesSecforc:
     Y_MOMENT = "y_moment"
     Z_MOMENT = "z_moment"
     TOTAL_MOMENT = "total_moment"
+
+class DataChannelTypesSbtout:
+    """ """
+    BELT_FORCE = "belt_force"
+    BELT_LENGTH = "belt_length"
